/********************************************************* {COPYRIGHT-TOP} ***
 * Licensed Materials - Property of IBM
 * 5725-Z22, 5725-Z63, 5725-U33, 5725-Z63
 *
 * (C) Copyright IBM Corporation 2017
 *
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ********************************************************** {COPYRIGHT-END} **/
module.exports = [
  '$http',
  '$log',
  '$q',
  function ConfigServices(
    $http,
    $log,
    $q
  ){
  'use strict';

  var svc = this;

  svc.inspectPath = function(path) {
    var def = $q.defer();

    $http.get('/apic-config/inspectPath', {
      params: {
        path: path
      }
    })
      .then(function(result){
        var project = result.data;
        project.displayName = project.name || project.basePath;

        def.resolve(project);
      })
      .catch(function(err){
        $log.error(err);
        def.reject(err);
      });

    return def.promise;
  };

  svc.getId = function(){
    return $http.get('/apic-config/id');
  };

  svc.getByName = function(name){
    return $http.get('/apic-config/get?key='+name);
  };

  svc.getEnableAnalytics = function(){
    return svc.getByName('enable-analytics');
  };

  svc.getAgreementByName = function(name){
    return $http.get('/apic-config/agreements/'+name);
  };

  svc.getAnalyticsAgreement = function(){
    return svc.getAgreementByName('analytics');
  };

  svc.getReminderDate = function(){
    return svc.getByName('analytics-reminder');
  };

  svc.setReminderDate = function(date){
    var global = true;

    return svc.setByName('analytics-reminder', date, global);
  };

  svc.setByName = function(name, value, global){
    global = !!global;

    return $http.post('/apic-config/set', {
      key: name,
      value: value,
      global: global
    });
  };

  svc.deleteByName = function(name, global){
    global = !!global;

    return $http.post('/apic-config/delete', {
      key: name,
      global: global
    });
  };

  svc.getAnalyticsKey = function(){
    return $http.get('/apic-config/keys/analytics');
  };

  svc.setEnableAnalytics = function(enabled){
    var global = true;

    return svc.setByName('enable-analytics', enabled, global);
  };

  return svc;
}];

